#include <QDebug>
#include <QEvent>

#include "alarmbusitem.h"

/* ok, warning, alarm types */
AlarmMonitor *DeviceAlarms::addTripplet(const QString &description, const QString &busitemPathAlarm,
						const QString &busitemSetting, const QString &busitemPathValue)
{
	AlarmMonitor *ret = new AlarmMonitor(mService, AlarmMonitor::REGULAR, busitemPathAlarm, busitemSetting, busitemPathValue, this);
	ret->setDescription(description);
	mAlarms.append(ret);
	return ret;
}

void DeviceAlarms::addVebusError(const QString &busitemPathAlarm)
{
	mAlarms.append(new AlarmMonitor(mService, AlarmMonitor::VEBUS_ERROR, busitemPathAlarm, "", "", this));
}

void DeviceAlarms::addLynxIonError(const QString &busitemPathAlarm)
{
	mAlarms.append(new AlarmMonitor(mService, AlarmMonitor::LYNXION_ERROR, busitemPathAlarm, "", "", this));
}

void DeviceAlarms::addChargerError(const QString &busitemPathAlarm)
{
	mAlarms.append(new AlarmMonitor(mService, AlarmMonitor::CHARGER_ERROR, busitemPathAlarm, "", "", this));
}

DeviceAlarms *DeviceAlarms::createBatteryAlarms(DBusService *service, NotificationCenter *noticationCenter)
{
	DeviceAlarms *alarms = new DeviceAlarms(service, noticationCenter);

	alarms->addTripplet(tr("Low voltage"),				"/Alarms/LowVoltage",			"",		"/Dc/0/Voltage");
	alarms->addTripplet(tr("High voltage"),				"/Alarms/HighVoltage",			"",		"/Dc/0/Voltage");
	alarms->addTripplet(tr("Over current"),				"/Alarms/OverCurrent",			"",		"/Dc/0/Current");
	alarms->addTripplet(tr("High charge current"),		"/Alarms/HighChargeCurrent",	"",		"/Dc/0/Current");
	alarms->addTripplet(tr("High discharge current"),	"/Alarms/HighDischargeCurrent",	"",		"/Dc/0/Current");
	alarms->addTripplet(tr("Low SOC"),					"/Alarms/LowSoc",				"",		"/Soc");
	alarms->addTripplet(tr("State of health"),			"/Alarms/StateOfHealth",		"",		"/Soh");
	alarms->addTripplet(tr("Low starter voltage"),		"/Alarms/LowStarterVoltage",	"",		"/Dc/1/Voltage");
	alarms->addTripplet(tr("High starter voltage"),		"/Alarms/HighStarterVoltage",	"",		"/Dc/1/Voltage");
	alarms->addTripplet(tr("Low temperature"),			"/Alarms/LowTemperature",		"",		"/Dc/0/Temperature");
	alarms->addTripplet(tr("High Temperature"),			"/Alarms/HighTemperature",		"",		"/Dc/0/Temperature");
	alarms->addTripplet(tr("Mid-point voltage"),		"/Alarms/MidVoltage",			"",		"/Dc/0/MidVoltageDeviation");
	alarms->addTripplet(tr("Low-fused voltage"),		"/Alarms/LowFusedVoltage",		"",		"/Dc/1/Voltage");
	alarms->addTripplet(tr("High-fused voltage"),		"/Alarms/HighFusedVoltage",		"",		"/Dc/1/Voltage");
	alarms->addTripplet(tr("Fuse blown"),				"/Alarms/FuseBlown",			"",		"");
	alarms->addTripplet(tr("High internal temperature"),"/Alarms/HighInternalTemperature","",	"");
	alarms->addTripplet(tr("Zinc pump"),				"/Alarms/ZincPump",				"",		"");
	alarms->addTripplet(tr("Bromide pump"),				"/Alarms/BromidePump",			"",		"");
	alarms->addTripplet(tr("Leak sensors"),				"/Alarms/LeakSensors",			"",		"");
	alarms->addTripplet(tr("Internal failure"),			"/Alarms/InternalFailure",		"",		"");
	alarms->addTripplet(tr("Electric board"),			"/Alarms/ElectricBoard",		"",		"");
	alarms->addTripplet(tr("Battery temperature sensor"),	"/Alarms/BatteryTemperatureSensor",	"", "");
	alarms->addTripplet(tr("Air temperature sensor"),	"/Alarms/AirTemperatureSensor",	"",		"");
	alarms->addTripplet(tr("Leak 1 trip"),				"/Alarms/Leak1Trip",			"",		"");
	alarms->addTripplet(tr("Leak 2 trip"),				"/Alarms/Leak2Trip",			"",		"");
	alarms->addTripplet(tr("Unknown"),					"/Alarms/Unknown",				"",		"");
	alarms->addTripplet(tr("Cell imbalance"),			"/Alarms/CellImbalance",		"",		"");

	return alarms;
}

DeviceAlarms *DeviceAlarms::createSolarChargerAlarms(DBusService *service, NotificationCenter *noticationCenter)
{
	DeviceAlarms *alarms = new DeviceAlarms(service, noticationCenter);

	alarms->addTripplet(tr("Low battery voltage"),		"/Alarms/LowVoltage",			"",		"/Dc/0/Voltage");
	alarms->addTripplet(tr("High batteryvoltage"),		"/Alarms/HighVoltage",			"",		"/Dc/0/Voltage");
	alarms->addChargerError("/ErrorCode");

	return alarms;
}

DeviceAlarms *DeviceAlarms::createAcChargerAlarms(DBusService *service, NotificationCenter *noticationCenter)
{
	DeviceAlarms *alarms = new DeviceAlarms(service, noticationCenter);

	alarms->addTripplet(tr("Low battery voltage"),		"/Alarms/LowVoltage",			"",		"/Dc/0/Voltage");
	alarms->addTripplet(tr("High battery voltage"),		"/Alarms/HighVoltage",			"",		"/Dc/0/Voltage");
	alarms->addChargerError("/ErrorCode");

	return alarms;
}

DeviceAlarms *DeviceAlarms::createLynxIonAlarms(DBusService *service, NotificationCenter *noticationCenter)
{
	DeviceAlarms *alarms = new DeviceAlarms(service, noticationCenter);

	alarms->addLynxIonError("/Error");

	return alarms;
}

DeviceAlarms *DeviceAlarms::createInverterAlarms(DBusService *service, NotificationCenter *noticationCenter)
{
	DeviceAlarms *alarms = new DeviceAlarms(service, noticationCenter);

	alarms->addTripplet(tr("Low battery voltage"),		"/Alarms/LowVoltage",			"",		"/Dc/0/Voltage");
	alarms->addTripplet(tr("High battery voltage"),		"/Alarms/HighVoltage",			"",		"/Dc/0/Voltage");
	alarms->addTripplet(tr("Low AC voltage"),			"/Alarms/LowVoltageAcOut",		"",		"/Ac/Out/L1/V");
	alarms->addTripplet(tr("High AC voltage"),			"/Alarms/HighVoltageAcOut",		"",		"/Ac/Out/L1/V");
	alarms->addTripplet(tr("Low temperature"),			"/Alarms/LowTemperature",		"",		"");
	alarms->addTripplet(tr("High temperature"),			"/Alarms/HighTemperature",		"",		"");
	alarms->addTripplet(tr("Inverter overload"),		"/Alarms/Overload",				"",		"/Ac/Out/L1/I");
	alarms->addTripplet(tr("High DC ripple"),			"/Alarms/Ripple",				"",		"");

	return alarms;
}

DeviceAlarms *DeviceAlarms::createSystemCalcAlarms(DBusService *service, NotificationCenter *noticationCenter)
{
	DeviceAlarms *alarms = new DeviceAlarms(service, noticationCenter);

	alarms->addTripplet(tr("Circuit breaker tripped"), "/Dc/Battery/Alarms/CircuitBreakerTripped", "", "");

	return alarms;
}

AlarmBusitem::AlarmBusitem(DBusServices *services, NotificationCenter *notificationCenter) :
	QObject(services),
	mNotificationCenter(notificationCenter)
{
	connect(services, SIGNAL(dbusServiceFound(DBusService*)), SLOT(dbusServiceFound(DBusService*)));
}

void AlarmBusitem::dbusServiceFound(DBusService *service)
{
	switch (service->getType())
	{
	case DBusService::DBUS_SERVICE_BATTERY:
		DeviceAlarms::createBatteryAlarms(service, mNotificationCenter);
		break;
	case DBusService::DBUS_SERVICE_MULTI:
		new VebusAlarms(service, mNotificationCenter);
		break;
	case DBusService::DBUS_SERVICE_SOLAR_CHARGER:
		DeviceAlarms::createSolarChargerAlarms(service, mNotificationCenter);
		break;
	case DBusService::DBUS_SERVICE_BMS:
		DeviceAlarms::createLynxIonAlarms(service, mNotificationCenter);
		break;
	case DBusService::DBUS_SERVICE_AC_CHARGER:
		DeviceAlarms::createAcChargerAlarms(service, mNotificationCenter);
		break;
	case DBusService::DBUS_SERVICE_INVERTER:
		DeviceAlarms::createInverterAlarms(service, mNotificationCenter);
		break;
	case DBusService::DBUS_SERVICE_SYSTEM_CALC:
		DeviceAlarms::createSystemCalcAlarms(service, mNotificationCenter);
		break;
	default:
		;
	}
}

VebusAlarms::VebusAlarms(DBusService *service, NotificationCenter *noticationCenter) : DeviceAlarms(service, noticationCenter)
{
	mNumberOfPhases = service->item("/Ac/NumberOfPhases");
	mNumberOfPhases->getValueAndChanges(this, SLOT(numberOfPhasesChanged(VeQItem *, QVariant)));
}

void VebusAlarms::init(bool single)
{
	addVebusError("/VebusError");
	addTripplet(tr("Temperature sense error"),	"/Alarms/TemperatureSensor",	"/Vebus/TemperatureSenseError");
	addTripplet(tr("Voltage sense error"),		"/Alarms/VoltageSensor",		"/Vebus/VoltageSenseError");

	// backwards compatible, the CAN-bus sends these e.g.
	addTripplet(tr("Low Battery"),				"/Alarms/LowBattery",			"/Vebus/LowBattery",			"/Dc/0/Voltage");
	addTripplet(tr("High Temperature"),			"/Alarms/HighTemperature",		"/Vebus/HighTemperature");
	addTripplet(tr("Inverter overload"),		"/Alarms/Overload",				"/Vebus/InverterOverload");
	addTripplet(tr("High DC ripple"),			"/Alarms/Ripple",				"/Vebus/HighDcRipple");

	// Phase 1 (note the description depends on the number of phases!)
	lowBatTextL1Alarm = addTripplet(lowBatTextL1(single),						"/Alarms/L1/LowBattery",		"/Vebus/LowBattery",			"/Dc/0/Voltage");
	highTempTextL1Alarm = addTripplet(highTempTextL1(single),					"/Alarms/L1/HighTemperature",	"/Vebus/HighTemperature");
	inverterOverloadTextL1Alarm = addTripplet(inverterOverloadTextL1(single),	"/Alarms/L1/Overload",			"/Vebus/InverterOverload");
	highDcRippleTextL1Alarm = addTripplet(highDcRippleTextL1(single),			"/Alarms/L1/Ripple",			"/Vebus/HighDcRipple");
	// Phase 2
	addTripplet(tr("Low Battery on L2"),		"/Alarms/L2/LowBattery",		"/Vebus/LowBattery",			"/Dc/0/Voltage");
	addTripplet(tr("High Temperature on L2"),	"/Alarms/L2/HighTemperature",	"/Vebus/HighTemperature");
	addTripplet(tr("Inverter overload on L2"),	"/Alarms/L2/Overload",			"/Vebus/InverterOverload");
	addTripplet(tr("High DC ripple on L2"),		"/Alarms/L2/Ripple",			"/Vebus/HighDcRipple");
	// Phase 3
	addTripplet(tr("Low Battery on L3"),		"/Alarms/L3/LowBattery",		"/Vebus/LowBattery",			"/Dc/0/Voltage");
	addTripplet(tr("High Temperature on L3"),	"/Alarms/L3/HighTemperature",	"/Vebus/HighTemperature");
	addTripplet(tr("Inverter overload on L3"),	"/Alarms/L3/Overload",			"/Vebus/InverterOverload");
	addTripplet(tr("High DC ripple on L3"),		"/Alarms/L3/Ripple",			"/Vebus/HighDcRipple");
}

void VebusAlarms::update(bool single)
{
	// update phase 1 (note the description depends on the number of phases!)
	lowBatTextL1Alarm->setDescription(lowBatTextL1(single));
	highTempTextL1Alarm->setDescription(highTempTextL1(single));
	inverterOverloadTextL1Alarm->setDescription(inverterOverloadTextL1(single));
	highDcRippleTextL1Alarm->setDescription(highDcRippleTextL1(single));
}

void VebusAlarms::numberOfPhasesChanged(VeQItem *item, QVariant value)
{
	Q_UNUSED(item);

	if (!value.isValid())
		return;

	bool singlePhase = value.toInt() == 1;
	if (mAlarms.empty()) {
		init(singlePhase);
	} else {
		update(singlePhase);
	}
}
