#ifndef ALARMMONITOR_H
#define ALARMMONITOR_H

#include <QObject>
#include <velib/qt/ve_qitem.hpp>

#include "notification.h"
#include "notification_center.h"
#include <dbus_service.h>

class DeviceAlarms;

class AlarmMonitor : public QObject
{
	Q_OBJECT

public:
	enum Type {
		REGULAR,
		VEBUS_ERROR,
		LYNXION_ERROR,
		CHARGER_ERROR
	};

	enum Enabled {
		NO_ALARM,
		ALARM_ONLY,
		ALARM_AND_WARNING
	};

	enum DbusAlarm {
		DBUS_NO_ERROR,
		DBUS_WARNING,
		DBUS_ERROR
	};

	explicit AlarmMonitor(DBusService *service, Type type, const QString &busitemPathAlarm,
						  const QString &busitemSetting = "", const QString &busitemPathValue = "", DeviceAlarms *parent = 0);

	~AlarmMonitor();
	void setDescription(QString description) {mDescription = description; }

private slots:
	void notificationDestroyed();
	void settingChanged(VeQItem * item, QVariant var);
	void updateAlarm(VeQItem *item, QVariant var);

private:
	DBusService *mService;
	Notification *mNotification;
	VeQItem *mBusitemValue;
	VeQItem *mAlarmTrigger;
	Type mType;
	QString mDescription;
	Enabled mEnabledNotifications;
	DeviceAlarms *mDeviceAlarms;

	bool mustBeShown(DbusAlarm alarm);
	void addNotification(Notification::Type type);
};

#endif
