#ifndef _APPLICATION_HPP_
#define _APPLICATION_HPP_

#include <QApplication>
#include <QTranslator>
#include <QsLog.h>

#include <velib/qt/ve_qitem.hpp>

#include <arguments.h>
#include <platform/ve_platform.hpp>

class Application : public QApplication
{
	Q_OBJECT
	Q_PROPERTY(bool withQwacs READ getWithQwacs CONSTANT)
	Q_PROPERTY(bool withScreenSaver READ getWithScreenSaver CONSTANT)
	Q_PROPERTY(VeQItem *veQItemRoot READ getVeQItemRoot)

	enum CompileFlags {
		CompiledFlagNone = 0,
		CompiledFlagQwacs = 1,
		CompiledFlagScreenSaver = 2
	};

public:
	Application(int &argc, char **argv);
	bool getWithQwacs() { return mCompileFlags & CompiledFlagQwacs; }
	bool getWithScreenSaver() { return mCompileFlags & CompiledFlagScreenSaver; }
	VeQItem *getVeQItemRoot() { return VeQItems::getRoot(); }

private slots:
	void onUpAndRunning();

private:
	Arguments mArguments;
	VePlatform *mPlatform;
	QTranslator mTranslator;
	VeQuickView *mDefaultView;
	int mCompileFlags;

	void loadTranslation();
	void initArguments();
	void initLogger(QsLogging::Level logLevel);
	void run();
	void setCompileFlags();
};

#endif
