#include <qstringlist.h>
#include <QEvent>

#include <dbus_service.h>

DBusService::DBusService(const QString &serviceName, DbusServiceType serviceType, QObject *parent) :
	QObject(parent),
	mServiceType(serviceType),
	mDbusServiceName(serviceName),
	mConnected(true),
	mServiceItem(VeQItems::getRoot()->itemGetOrCreate("dbus/" + serviceName)) // FIXME
{
	setDefaultDescription();
	item("ProductName")->getValueAndChanges(this, SLOT(productNameChanged(VeQItem*, QVariant)));
}

QString DBusService::getName() const
{
	return mDbusServiceName;
}

void DBusService::setDefaultDescription()
{
	switch (mServiceType)
	{
	case DBUS_SERVICE_MULTI:
		mDescription = tr("Multi");
		break;
	case DBUS_SERVICE_BATTERY:
		mDescription = tr("Battery Monitor");
		break;
	case DBUS_SERVICE_SOLAR_CHARGER:
		mDescription = tr("BlueSolar Charger");
		break;
	case DBUS_SERVICE_BMS:
		mDescription = tr("BMS");
		break;
	case DBUS_SERVICE_PV_INVERTER:
		mDescription = tr("PV-Inverter");
		break;
	case DBUS_SERVICE_AC_CHARGER:
		mDescription = tr("Charger");
		break;
	case DBUS_SERVICE_TANK:
		mDescription = tr("Tank");
		break;
	case DBUS_SERVICE_GRIDMETER:
		mDescription = tr("Grid meter");
		break;
	case DBUS_SERVICE_GENSET:
		mDescription = tr("Generator");
		break;
	case DBUS_SERVICE_MOTOR_DRIVE:
		mDescription = tr("Motor Drive");
		break;
	case DBUS_SERVICE_INVERTER:
		mDescription = tr("Inverter");
		break;
	case DBUS_SERVICE_TEMPERATURE_SENSOR:
		mDescription = tr("Temperature sensor");
		break;
	default:
		mDescription = tr("Unknown Device");
	}
}

void DBusService::productNameChanged(VeQItem *item, QVariant var)
{
	Q_UNUSED(item);

	if (var.isValid())
		setDescription(var.toString());
}

DBusService *DBusService::createInstance(QString name)
{
	if (!name.startsWith("com.victronenergy."))
		return 0;

	QStringList elements = name.split(".");
	if (elements.count() < 3)
		return 0;

	QString type = elements[2];

	if (type == "battery" || type == "zbmnode")
		return new DBusService(name, DBUS_SERVICE_BATTERY);

	if (type == "vebus")
		return new DBusService(name, DBUS_SERVICE_MULTI);

	if (type == "solarcharger")
		return new DBusService(name, DBUS_SERVICE_SOLAR_CHARGER);

	if (type == "bms")
		return new DBusService(name, DBUS_SERVICE_BMS);

	if (type == "pvinverter")
		return new DBusService(name, DBUS_SERVICE_PV_INVERTER);

	if (type == "charger")
		return new DBusService(name, DBUS_SERVICE_AC_CHARGER);

	if (type == "tank")
		return new DBusService(name, DBUS_SERVICE_TANK);

	if (type == "grid")
		return new DBusService(name, DBUS_SERVICE_GRIDMETER);

	if (type == "genset")
		return new DBusService(name, DBUS_SERVICE_GENSET);

	if (type == "motordrive")
		return new DBusService(name, DBUS_SERVICE_MOTOR_DRIVE);

	if (type == "inverter")
		return new DBusService(name, DBUS_SERVICE_INVERTER);

	if (type == "system")
		return new DBusService(name, DBUS_SERVICE_SYSTEM_CALC);

	if (type == "temperature")
		return new DBusService(name, DBUS_SERVICE_TEMPERATURE_SENSOR);

	return 0;
}

bool DBusService::event(QEvent *event)
{
	if (event->type() == QEvent::LanguageChange)
		foreach (QObject *obj, children())
			obj->event(event);

	return QObject::event(event);
}
