#ifndef DBUS_SERVICE_H
#define DBUS_SERVICE_H

#include <QObject>
#include <QString>

#include <velib/qt/ve_qitem.hpp>

class DBusService : public QObject
{
	Q_OBJECT
	Q_PROPERTY(QString description READ getDescription WRITE setDescription NOTIFY descriptionChanged)
	Q_PROPERTY(QString name READ getName)
	Q_PROPERTY(DbusServiceType type READ getType)
	Q_ENUMS(DbusServiceType)
	Q_PROPERTY(bool connected READ getConnected NOTIFY connectedChanged)

public:
	enum DbusServiceType {
		DBUS_SERVICE_MULTI,
		DBUS_SERVICE_BATTERY,
		DBUS_SERVICE_SOLAR_CHARGER,
		DBUS_SERVICE_BMS,
		DBUS_SERVICE_PV_INVERTER,
		DBUS_SERVICE_AC_CHARGER,
		DBUS_SERVICE_TANK,
		DBUS_SERVICE_GRIDMETER,
		DBUS_SERVICE_GENSET,
		DBUS_SERVICE_MOTOR_DRIVE,
		DBUS_SERVICE_INVERTER,
		DBUS_SERVICE_SYSTEM_CALC,
		DBUS_SERVICE_TEMPERATURE_SENSOR
	};

	DBusService(const QString &name, DbusServiceType serviceType, QObject *parent = 0);
	QString getName() const;
	virtual bool event(QEvent *event);

	void setDescription(QString description) {
		if (mDescription == description)
			return;
		mDescription = description;
		emit descriptionChanged();
	}
	QString getDescription() {return mDescription;}

	static DBusService *createInstance(QString name);

	DbusServiceType getType() {return mServiceType; }

	Q_INVOKABLE QString path(QString str) {
		return mDbusServiceName + str;
	}
	inline VeQItem *item(const QString &id) { return mServiceItem->itemGetOrCreate(id); }

	bool getConnected() { return mConnected; }
	void setConnected(bool connected) {
		if (mConnected != connected) {
			mConnected = connected;
			emit connectedChanged();
		}
	}

signals:
	void descriptionChanged();
	void connectedChanged();

public slots:
	void productNameChanged(VeQItem *item, QVariant var);

private:
	void setDefaultDescription();

	DbusServiceType mServiceType;
	QString mDbusServiceName;
	QString mDescription;
	bool mConnected;
	VeQItem *mServiceItem;
};

#endif
