//#define QS_LOG_DISABLE
#include "QsLog.h"
#include <QEvent>

#include <dbus_service.h>
#include <dbus_services.h>

#include <velib/qt/v_busitems.h>

void DBusServices::initialScan()
{
	QDBusConnectionInterface *interface = VBusItems::getConnection().interface();
	QStringList serviceNames = interface->registeredServiceNames();
	connect(interface, SIGNAL(serviceOwnerChanged(QString,QString,QString)), this, SLOT(serviceOwnerChanged(QString,QString,QString)));

	foreach (QString name, serviceNames)
		processServiceName(name);
}

void DBusServices::processServiceName(QString name)
{
	DBusService *service;

	service = mServices.value(name);
	if (service) {
		service->setConnected(true);
		emit dbusServiceConnected(service);
		return;
	}

	service = DBusService::createInstance(name);
	if (service == 0)
		return;
	service->setParent(this);
	mServices.insert(name, service);
	emit dbusServiceFound(service);
	emit dbusServiceConnected(service);
}

void DBusServices::serviceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner)
{
	if (name.size() > 0 && name.at(0) != ':') {
		QLOG_TRACE() << "serviceOwnerChanged" << name << oldOwner << newOwner;
	}

	if (newOwner != "") {
		processServiceName(name);
	} else {
		DBusService *service = mServices.value(name);
		if (service) {
			service->setConnected(false);
			emit dbusServiceDisconnected(service);
		}
	}
}

bool DBusServices::event(QEvent *event)
{
	if (event->type() == QEvent::LanguageChange)
		foreach (QObject *obj, children())
			obj->event(event);

	return QObject::event(event);
}
