#ifndef NOTIFICATION_H
#define NOTIFICATION_H

#include <QObject>
#include <QDateTime>

class Notification : public QObject
{
	Q_OBJECT
	Q_PROPERTY(bool acknowledged READ acknowledged WRITE setAcknowledged NOTIFY acknowledgedChanged)
	Q_PROPERTY(bool active READ active WRITE setActive NOTIFY activeChanged)
	Q_PROPERTY(Type type READ type WRITE setType NOTIFY typeChanged)
	Q_PROPERTY(QString typeAsString READ getTypeAsString NOTIFY typeChanged)
	Q_PROPERTY(QString serviceName READ serviceName NOTIFY servicenameChanged)
	Q_PROPERTY(QDateTime dateTime READ dateTime NOTIFY dateTimeChanged)
	Q_PROPERTY(QString description READ description NOTIFY descriptionChanged)
	Q_PROPERTY(QString value READ value WRITE setValue NOTIFY valueChanged)

public:
	enum Type {
		WARNING,
		ALARM,
		NOTIFICATION
	};

	Notification(Type type, const QString &devicename,
					const QString &description, const QString &value, QObject *parent = 0);

	bool acknowledged() const { return mAcknowledged; }

	void setAcknowledged(const bool acknowledged) {
		if (acknowledged != mAcknowledged) {
			mAcknowledged = acknowledged;
			emit acknowledgedChanged(this);
		}
	}

	/*
	 * By default notifications are active. Alarms condition can disappear
	 * however while the alarm has not been acknowledged yet. In that case
	 * active becomes false.
	 */
	bool active() const { return mActive; }

	void setActive(const bool active) {
		if (active != mActive) {
			mActive = active;
			emit activeChanged(this);
		}
	}

	Notification::Type type() { return mType; }

	void setType(Type type) {
		if (type != mType) {
			mType = type;
			emit typeChanged(this);
		}
	}

	QString serviceName() const { return mDevicename; }
	QDateTime dateTime() const { return mDateTime; }
	QString description() const { return mDescription; }

	QString value() const { return mValue; }

	void setValue(const QString &value) {
		if (value != mValue) {
			mValue = value;
			emit valueChanged(this);
		}
	}

	QString getTypeAsString() const;

signals:
	void acknowledgedChanged(Notification *notification);
	void activeChanged(Notification *notification);
	void typeChanged(Notification *notification);
	void servicenameChanged(Notification *notification);
	void dateTimeChanged(Notification *notification);
	void descriptionChanged(Notification *notification);
	void valueChanged(Notification *notification);

private:
	bool mAcknowledged;
	bool mActive;
	Type mType;
	QString mDevicename;
	QDateTime mDateTime;
	QString mDescription;
	QString mValue;
};

#endif
