#include <QTimer>
#include <QDebug>

#include "notification_center.h"

NotificationCenter::NotificationCenter(QObject *parent) :
	QObject(parent),
	mAlert(false), mAlarm(false), mMaxNotifications(20)
{
	// QTimer::singleShot(9000, this, SLOT(test())); // TEST
}

void NotificationCenter::acknowledgedAll()
{
	foreach (QObject *obj, mNotifications) {
		Notification *notification = qobject_cast<Notification *>(obj);
		notification->setAcknowledged(true);
	}
	setAlert(false);
	setAlarm(false);
}

void NotificationCenter::updateAlert()
{
	foreach (QObject *obj, mNotifications) {
		Notification *notification = qobject_cast<Notification *>(obj);
		if (!notification->acknowledged()) {
			setAlert(true);
			return;
		}
	}

	setAlert(false);
}

void NotificationCenter::updateAlarm()
{
	foreach (QObject *obj, mNotifications) {
		Notification *notification = qobject_cast<Notification *>(obj);
		if (notification->type() == Notification::ALARM && notification->active() && !notification->acknowledged()) {
			setAlarm(true);
			return;
		}
	}

	setAlarm(false);
}

Notification *NotificationCenter::addNotification(Notification::Type type, const QString &devicename,
													const QString &description, const QString &value)
{
	Notification *notification;

	if (mNotifications.size() >= mMaxNotifications) {
		notification = qobject_cast<Notification *>(mNotifications.last());
		removeNotification(notification);
	}

	notification = new Notification(type, devicename, description, value, this);
	mNotifications.insert(0, notification);
	connect(notification, SIGNAL(activeChanged(Notification *)), this, SLOT(activeChanged(Notification *)));
	connect(notification, SIGNAL(typeChanged(Notification *)), this, SLOT(typeChanged(Notification *)));
	emit notificationsChanged();

	if (notification->type() == Notification::ALARM)
		setAlarm(true);

	if (notification->active())
		setAlert(true);

	return notification;
}

void NotificationCenter::removeNotification(Notification *notification)
{
	notification->setActive(false);
	mNotifications.removeOne(notification);
	emit notificationsChanged();
	delete notification;

	updateAlarm();
	updateAlert();
}

void NotificationCenter::activeChanged(Notification *notification)
{
	Q_UNUSED(notification);
	updateAlarm();
	updateAlert();
}

void NotificationCenter::typeChanged(Notification *notification)
{
	if (notification->type() == Notification::ALARM) {
		setAlarm(true);
		setAlert(true);
	}
}

void NotificationCenter::test()
{
	qDebug() << "NotificationCenter test()";
	addNotification(Notification::ALARM, "Multi", "VeBus error", "value");
	addNotification(Notification::NOTIFICATION, "Multi", "VeBus error", "value");
}
