#ifndef NOTIFICATION_CENTER_H
#define NOTIFICATION_CENTER_H

#include <QObject>

#include "notification.h"

class NotificationCenter : public QObject
{
	Q_OBJECT
	Q_PROPERTY(bool alert READ getAlert NOTIFY alertChanged)
	Q_PROPERTY(bool alarm READ getAlarm NOTIFY alarmChanged)
	Q_PROPERTY(QList<QObject*> notifications READ getNotifications NOTIFY notificationsChanged)

public:
	NotificationCenter(QObject *parent = 0);

	bool getAlert() const { return mAlert; }
	bool getAlarm() const { return mAlarm; }
	QList<QObject *> getNotifications() const { return mNotifications; }

	Notification* addNotification(Notification::Type type, const QString &devicename,
									const QString &description, const QString &value);
	void removeNotification(Notification *notification);

signals:
	void alertChanged();
	void alarmChanged();
	void notificationsChanged();

public slots:
	void acknowledgedAll();

private slots:
	void activeChanged(Notification *notification);
	void typeChanged(Notification *notification);
	void test();

private:
	bool mAlert;
	bool mAlarm;
	QList<QObject*> mNotifications;
	int mMaxNotifications;

	void updateAlert();
	void updateAlarm();

	void setAlert(bool alert) {
		if (alert != mAlert) {
			mAlert = alert;
			emit alertChanged();
		}
	}

	void setAlarm(bool alarm) {
		if (alarm != mAlarm) {
			mAlarm = alarm;
			emit alarmChanged();
		}
	}
};

#endif
