#include <QtDeclarative>

#include <velib/qt/ve_qitem.hpp>
#include <velib/qt/ve_qitem_table_model.hpp>

#include <application.hpp>
#include <platform/ve_platform.hpp>
#include <platform/ve_platform_ccgx.hpp>

#ifdef CFG_WITH_DBUS
#include <velib/qt/v_busitems.h>
#include <velib/qt/ve_qitems_dbus.hpp>
#include <dbus_service.h>
#endif

#include <ve_qitem_table_widget.hpp>
#include <ve_quick_item_model.hpp>

VePlatform *VePlatform::create(Application &app)
{
	static VePlatform *platform;

	if (platform)
		return platform;

	platform = new VePlatformCcgx(app);

	return platform;
}

void VePlatform::qmlRegister(VeQuickView *view)
{
	Q_UNUSED(view);

	qmlRegisterUncreatableType<VeQItem>("com.victron.velib", 1, 0, "VeQItem", "VeQItem cannot be created in qml (at least not for now),"
																			  " create a VeQuickItem, it will look one up for you..");
	qmlRegisterType<VeQuickItem>("com.victron.velib", 1, 0, "VeQuickItem");
	view->rootContext()->setContextProperty("VePlatform", this);

	qmlRegisterType<VeQuickItemModelBuilder>("com.victron.velib", 1, 0, "VeQuickItemModelBuilder");

#ifdef CFG_WITH_DBUS
	VeQItemDbusProducer *producer = new VeQItemDbusProducer(mApp.getVeQItemRoot(), "dbus");
	producer->open(VBusItems::getConnection());

	qmlRegisterUncreatableType<DBusService>("com.victron.velib", 1, 0, "DBusService", "Dbus services are found, not created");
#endif

	mSettings = new VeQItemSettings(producer->services(), QString("com.victronenergy.settings"));
}
