#ifndef QWACS_H
#define QWACS_H

#include <QObject>
#include <QtDeclarative>
#include <QStringList>

#include <qwacs/busitem_cons.h>
#include <qwacs/qwacs_manager_interface.h>
#include <qwacs/qwacs_gateway_interface.h>

class Qwacs : public QObject
{
	Q_OBJECT
	//Q_PROPERTY(bool blink READ getBlink WRITE setBlink)
	//Q_PROPERTY(bool blinking READ getBlinking NOTIFY blinkingChanged)
	Q_PROPERTY(QStringList sensors READ getSensors NOTIFY sensorsChanged)
	Q_PROPERTY(QString version READ getVersion NOTIFY versionChanged)
	Q_PROPERTY(QString connected READ getConnected NOTIFY connectedChanged)
	Q_PROPERTY(QString hostname READ getHostname NOTIFY hostnameChanged)
	Q_PROPERTY(QString uplinkStatus READ getUplinkStatus NOTIFY uplinkStatusChanged)
	Q_PROPERTY(bool uplink READ getUplink WRITE setUplink)
	Q_PROPERTY(QString commonName READ getCommonName NOTIFY commonNameChanged)
	Q_PROPERTY(QString firmwareVersion READ getFirmwareVersion NOTIFY firmwareVersionChanged)
	Q_PROPERTY(QString archFlavVers READ getArchFlavVers NOTIFY archFlavVersChanged)
	Q_PROPERTY(QString serialNr READ getSerialNr NOTIFY serialNrChanged)
	Q_PROPERTY(QString partNr READ getPartNr NOTIFY partNrChanged)
	Q_PROPERTY(uint updays READ getUpdays NOTIFY updaysChanged)
	Q_PROPERTY(uint uphours READ getUphours NOTIFY uphoursChanged)
public:
	Qwacs(QDeclarativeContext *context, QObject *parent = 0);

	Q_INVOKABLE QString getPosition(const QString & id) { return mPositionsMap.value(id); }
	Q_INVOKABLE void setPosition(const QString & id, const QString & pos);
	Q_INVOKABLE void blinkOn(const QString & id);
	Q_INVOKABLE void blinkOff(const QString & id);

	QString getVersion() { return mVersion; }
	bool getConnected() { return mConnected; }
	//bool getBlink() { return mBlink; }
	//bool getBlinking() { return mBlinking; }
	//void setBlink(bool blink) { mBlink = blink; }
	QStringList getSensors();

	QString getHostname() { return mHostname; }
	QString getUplinkStatus() { if (mGateway) mGateway->getUplink(); return mUplinkStatus; }
	bool getUplink() { return mUplink; }
	void setUplink(bool enable) { if (mGateway) mGateway->setUplink(enable); }
	QString getCommonName() { return mCommonName; }
	QString getFirmwareVersion() { return mFirmwareVersion; }
	QString getArchFlavVers() { return mArchFlavVers; }
	QString getSerialNr() { return mSerialNr; }
	QString getPartNr() { return mPartNr; }
	uint getUpdays() { return mUpdays; }
	uint getUphours() { return mUphours; }
signals:

public slots:

private slots:
	void qwacsRegisterd(const QString serviceName);
	void qwacsUnregisterd(const QString serviceName);
	void gatewayFound(const QString &hostname);
	void sensorAdded(const QString &id);
	void managerChanges(const QVariantMap &changes);
	void gatewayChanges(const QVariantMap &changes);
	void uplinkStatus(const QString & status);

signals:
	void blinkingChanged();
	void positionChanged();
	void sensorsChanged();
	void versionChanged();
	void connectedChanged();
	void hostnameChanged();
	void uplinkStatusChanged();
	void commonNameChanged();
	void firmwareVersionChanged();
	void archFlavVersChanged();
	void serialNrChanged();
	void partNrChanged();
	void updaysChanged();
	void uphoursChanged();

private:
	void addSettingBusItem(const QString & item);

	//Manager
	bool mConnected;
	QString mVersion;
	// Gateway
	QString mHostname;
	QString mUplinkStatus;
	bool mUplink;
	QString mCommonName;
	QString mFirmwareVersion;
	QString mArchFlavVers;
	QString mSerialNr;
	QString mPartNr;
	uint mUpdays;
	uint mUphours;

	QDBusInterface *iface;

	//bool mBlink;
	//bool mBlinking;
	QStringList mSensorList;
	QMap<QString, QString> mPositionsMap;
	QMap<QString, BusItemCons *> mBusItemMap;
	QMap<QString, QString> mSensorsMap;
	QwacsManagerInterface * mManager;
	QwacsGatewayInterface * mGateway;
	QDBusServiceWatcher mQwacsWatcher;
	QDeclarativeContext *mContext;
};

#endif // QWACS_H
