#include <relay.h>
#include <QsLog.h>

Relay::Relay(QString relayPath, NotificationCenter *notificationCenter, QObject *parent) :
	QObject(parent),
	mNotificationCenter(notificationCenter),
	mIntendedState(false)
{
	mPolaritySetting = VeQItems::getRoot()->itemGetOrCreate("dbus/com.victronenergy.settings/Settings/Relay/Polarity");
	connect(mPolaritySetting, SIGNAL(valueChanged(VeQItem *, QVariant)), this, SLOT(updateRelayState()));

	mRelayFunction = VeQItems::getRoot()->itemGetOrCreate("dbus/com.victronenergy.settings/Settings/Relay/Function");
	connect(mRelayFunction, SIGNAL(valueChanged(VeQItem *, QVariant)), this, SLOT(updateRelayState()));

	mRelayState = VeQItems::getRoot()->itemGetOrCreate(relayPath);
	connect(mRelayState, SIGNAL(valueChanged(VeQItem *, QVariant)), this, SLOT(updateRelayState()));

	mPolaritySetting->getValue();
	mRelayFunction->getValue();
	mRelayState->getValue();

	connect(notificationCenter, SIGNAL(alarmChanged()), this, SLOT(alarmChanged()));
	setRelayOn(notificationCenter->getAlarm());
}

void Relay::updateRelayState()
{
	if (!mPolaritySetting->getValue().isValid() || !mRelayFunction->getValue().isValid())
		return;

	// only control the relay in alarm notification mode
	if (mRelayFunction->getValue() != 0)
		return;

	/* Set relay but allow software inversion */
	setHwState(mIntendedState ^ mPolaritySetting->getValue().toInt());
}

bool Relay::getRelayOn() const
{
	return mIntendedState;
}

void Relay::setRelayOn(bool on)
{
	if (mIntendedState == on)
		return;

	mIntendedState = on;
	emit relayOnChanged();
	updateRelayState();
}

/*
 * set the actual relay state
 *   true for closed / activated
 *   false for open / powered off state
 */
void Relay::setHwState(bool closed)
{
	if (!mRelayState->getValue().isValid())
		return;
	mRelayState->setValue(closed ? 1 : 0);
}

void Relay::alarmChanged()
{
	setRelayOn(mNotificationCenter->getAlarm());
}
