#include <QFile>

#include "screensaver.h"
#include "QsLog.h"

#ifndef CFG_BLANKFILE
	#ifdef CFG_WITHOUT_SCREENSAVER
		#define CFG_BLANKFILE "/dev/null"
	#else
		#define CFG_BLANKFILE "/sys/class/graphics/fb0/blank"
	#endif
#endif

ScreenSaver::ScreenSaver(QObject *parent) :
	QObject(parent)
{
	_screenSaved = false;
	_inhibitTime = 0;
	connect(&_screenSaverTimer, SIGNAL(timeout()), this, SLOT(_timer()));
}

void ScreenSaver::_timer()
{
	QLOG_TRACE() << "ScreenSaver::_timer";
	_screenOff();
	_screenSaverTimer.stop();
}

void ScreenSaver::enable()
{
	QLOG_TRACE() << "ScreenSaver::enable";
	_screenOn();
	trigger();
}

void ScreenSaver::disable()
{
	QLOG_TRACE() << "ScreenSaver::disable";
	_screenSaverTimer.stop();
	_screenOn();
}

void ScreenSaver::trigger()
{
	if (_inhibitTime > 0)
		_screenSaverTimer.start(_inhibitTime);
}

bool ScreenSaver::isSaved()
{
	return _screenSaved;
}

void ScreenSaver::setInhibitTime(unsigned int time)
{
	_inhibitTime = time*1000;
	if (_inhibitTime > 0) {
		_screenSaverTimer.start(_inhibitTime);
	}
	else {
		disable();
	}
}

int ScreenSaver::getInhitbitTime()
{
	return _inhibitTime/1000;
}

bool ScreenSaver::_screenOn()
{
	QLOG_TRACE() << "ScreenSaver::_screenOn";
	QFile blankFile(CFG_BLANKFILE);
	if (!blankFile.open(QIODevice::WriteOnly | QIODevice::Text))
		return false;
	blankFile.write("0");
	_screenSaved = false;
	blankFile.close();
	return true;
}

bool ScreenSaver::_screenOff()
{
	QLOG_TRACE() << "ScreenSaver::_screenOff";
	QFile blankFile(CFG_BLANKFILE);
	if (!blankFile.open(QIODevice::WriteOnly | QIODevice::Text))
		return false;
	blankFile.write("1");
	_screenSaved = true;
	blankFile.close();
	return true;
}
