#ifndef SERVICE_H
#define SERVICE_H

#include <QObject>
#include <QFile>
#include <QStringList>

#include <velib/qt/ve_qitem.hpp>
#include <ve_qitems_settings.hpp>

class Service : public QObject
{
	Q_OBJECT

public:
	Service(VeQItemSettings *settings, const QString &servicePath, const QString &dbusObjectPath, QObject *parent = 0);

	Service(VeQItemSettings *settings, const QString &servicePath, const QString &dbusObjectPath, int startValue, QObject *parent = 0);

	// Listens to multiple ObjectPaths. If one of them is True, the service will be started. Everytime
	// a value becomes True or False, the service will be restarted. Unless none of them is True any-more,
	// then the service will be stopped.
	Service(VeQItemSettings *settings, const QString &servicePath, const QList<QString> &dbusObjectPathList, QObject *parent = 0);

	void enableDaemon();
	void disableDaemon();
	void restartDaemon();

	// note: if services can ever be destroyed the destructor should clea this up
	void addConflictingService(Service *service);

protected:
	virtual void beforeDaemonStarted();
	virtual void afterDaemonStopped() {}
	void execSvc(QStringList arguments);
	void touch(QString fileName);

private slots:
	void dbusItemChanged(VeQItem *item, QVariant var);

private:
	const QString mServicePath;
	QList<QString> mDbusObjectPathList;
	const int mStartValue;
	QList<VeQItem*> mDbusItemList;
	QList<Service*> mConflictingServices;
	void init(VeQItemSettings *settings);
	inline bool daemonIsRunning() { return !QFile::exists(mServicePath + "/down"); }
};

// The console port is shared with ve.direct 1, so has some additional start / stop dependencies
class Console : public Service
{
	Q_OBJECT

public:
	Console(VeQItemSettings *settings, const QString &servicePath, const QString &dbusObjectPath, QObject *parent = 0) :
		Service(settings, servicePath, dbusObjectPath, parent)
	{
	}

protected:
	void beforeDaemonStarted() {
		Service::beforeDaemonStarted();

		// prevent serial starter from using ttyO0
		execSvc(QStringList() << "-t" << "/service/serial-starter");
		// stop using ttyO0
		execSvc(QStringList() << "-d" << "/service/*.ttyO0");
	}

	void afterDaemonStopped() {
		execSvc(QStringList() << "-t" << "/service/serial-starter");
	}
};

#endif
