//#define QS_LOG_DISABLE
#include "QsLog.h"
#include "timezone.h"
#include <ve_qitems_settings.hpp>

const QString service = "com.victronenergy.settings";
const QString objectPath = "/Settings/System/TimeZone";

TimeZone::TimeZone(VeQItemSettings *settings, QObject *parent) :
	QObject(parent),
	mCity("---")
{
	mBusitemValue = settings->root()->itemGetOrCreate("Settings/System/TimeZone");
	mBusitemValue->getValueAndChanges(this, SLOT(valueChanged(VeQItem*,QVariant)));
}

QString TimeZone::getCity()
{
	return mCity;
}

void TimeZone::setCity(QString city)
{
	mCity = city;
	mBusitemValue->setValue(mRegion + "/" + mCity);
	QLOG_TRACE() << "TimeZone::setCity: " << mRegion << "/" << mCity;
}

QString TimeZone::getRegion()
{
	return mRegion;
}

void TimeZone::setRegion(QString region)
{
	mRegion = region;
	QLOG_TRACE() << "TimeZone::setRegion: " << mRegion;
}

void TimeZone::valueChanged(VeQItem *item, QVariant var)
{
	QString zone;
	Q_UNUSED(item);
	Q_UNUSED(var);

	if (var.isValid()) {
		zone = var.toString();
		mRegion = zone.section("/", 0, 0);
		mCity = zone.section("/", 1);
	}

	setenv("TZ", zone.toLatin1(), 1);
	emit cityChanged();
	emit regionChanged();
	QLOG_TRACE() << "TimeZone::valueChanged: " << zone;
}
