#ifndef UPDATER_H
#define UPDATER_H

#include <QTimer>
#include <QProcess>
#include <QFileSystemWatcher>
#include <QtDeclarative>

#include "notification_center.h"

class Updater : public QObject
{
	Q_OBJECT
	Q_PROPERTY(bool running READ getRunning NOTIFY runningChanged)
	Q_PROPERTY(bool runningOffline READ getRunningOffline NOTIFY runningChanged)
	Q_PROPERTY(bool updateTriggered READ getUdateTriggered NOTIFY runningChanged)
	Q_PROPERTY(QString statusDescription READ getStatusDescription NOTIFY statusChanged)
	Q_PROPERTY(int status READ getStatus NOTIFY statusChanged)
	Q_PROPERTY(QString availableUpdate READ getAvailableUpdate NOTIFY availableUpdateChanged)
	Q_PROPERTY(QString offlineAvailableUpdate READ getOfflineAvailableUpdate NOTIFY availableUpdateChanged)
	Q_PROPERTY(QString availableUpdateBuild READ getAvailableUpdateBuild NOTIFY availableUpdateChanged)
	Q_PROPERTY(QString offlineAvailableUpdateBuild READ getOfflineAvailableUpdateBuild NOTIFY availableUpdateChanged)
	Q_PROPERTY(QVariantList availableRootfsVersions READ getAvailableRootfsVersions NOTIFY availableRootfsChanged)
	Q_PROPERTY(QVariantList availableRootfsBuildNumbers READ getAvailableRootfsBuildNumbers NOTIFY availableRootfsChanged)
	Q_PROPERTY(QString notification READ getNotification NOTIFY notificationChanged)

public:
	Updater(NotificationCenter *notificationCenter, QObject *parent = 0);

	QString getStatusDescription();
	int getStatus();
	QString getNotification();
	QString getAvailableUpdate();
	QString getAvailableUpdateBuild();
	QString getOfflineAvailableUpdate();
	QString getOfflineAvailableUpdateBuild();
	QVariantList getAvailableRootfsVersions();
	QVariantList getAvailableRootfsBuildNumbers();
	bool getRunning();
	bool getRunningOffline();
	bool getUdateTriggered();

	Q_INVOKABLE void doUpdate(bool offline=false);
	Q_INVOKABLE void checkUpdates(bool offline=false);
	Q_INVOKABLE void doOfflineUpdate();
	Q_INVOKABLE void checkOfflineUpdates();
	Q_INVOKABLE void switchVersion();

	enum UpdaterState {
		UpdateFileNotFound = -3,
		ErrorDuringUpdating = -2,
		ErrorDuringChecking = -1,
		Idle = 0,
		Checking = 1,
		DownloadingAndInstalling = 2,
		Rebooting = 3
	};

public slots:
	void updateScriptFinished(int exitCode, QProcess::ExitStatus exitStatus);

signals:
	void runningChanged();
	void statusChanged();
	void notificationChanged();
	void availableUpdateChanged();
	void availableRootfsChanged();

private slots:
	void updateAvailableChanged(const QString &str);
	void fileChanged(const QString &str);

private:
	void getUpdateInfoFromFile(const QString &str);
	void resetRunningStatus();
	bool mRunning;
	bool mRunningOffline;
	bool mUpdateTriggered;
	int mStatus;
	QString mStatusDescription;
	QString mNotification;
	QString mAvailableUpdate;
	QString mOfflineAvailableUpdate;
	QString mAvailableUpdateBuild;
	QString mOfflineAvailableUpdateBuild;
	QVariantList mAvailableRootfsVersions;
	QVariantList mAvailableRootfsBuildNumbers;
	QDeclarativeContext *mContext;
	QProcess mUpdateScript;
	QProcess mSetVersionScript;
	QFileSystemWatcher mUpdateWatcher;
	NotificationCenter *mNotificationCenter;
	void getRootfsInfoFromFile(const QString &str);
	void versionsInfoChanged(const QString &str);
	void touchFile(QString filePath);
	QMap<QString, QString> getVersionInfoFromLine(QString versionLine);
};

#endif
