#include <unistd.h>
#include "v_settings.h"
#include "QsLog.h"
#include <velib/qt/v_busitems.h>

typedef struct setting
{
	QString group;
	QString name;
	QVariant defaultValue;
	QString typeValue;
	QVariant min;
	QVariant max;
} SETTING;

SETTING settings[] =
{
	{"Gps", "Format", 0, "i", 0, 0},
	{"Gps", "SpeedUnit", "km/h", "s", 0, 0},
	{"Gui", "Brightness", 100, "i", 0, 0},
	{"Gui", "DisplayOff", 600, "i", 0, 0},
	{"Gui", "DemoMode", 0, "i", 0, 0},
	{"Gui", "StartWithMenuView", 0, "i", 0, 1},
	{"Gui", "DefaultOverview", "", "s", 0, 0},
	{"Gui", "MobileOverview", 0, "i", 0, 1},
	{"SystemSetup", "AcInput1", 1, "i", 0, 0},
	{"SystemSetup", "AcInput2", 2, "i", 0, 0},
	{"SystemSetup", "HasDcSystem", 0, "i", 0, 1},
	{"SystemSetup", "SystemName", "", "s", 0, 0},
	{"System", "AutoUpdate", 1, "i", 0, 0},
	{"System", "ReleaseType", 0, "i", 0, 0},
	{"System", "LogLevel", 2, "i", 0, 0},
	{"System", "TimeZone", "/UTC", "s", 0, 0},
	{"System", "AccessLevel", 1, "i", 0, 0},
	{"System", "RemoteSupport", 0, "i", 0, 0},
	{"System", "VolumeUnit", 0, "i", 0, 0},
	{"Alarm", "Audible", 1, "i", 0, 0},
	{"Alarm/Vebus", "LowBattery", 1, "i", 0, 0},
	{"Alarm/Vebus", "HighTemperature", 1, "i", 0, 0},
	{"Alarm/Vebus", "InverterOverload", 1, "i", 0, 0},
	{"Alarm/Vebus", "HighDcRipple", 1, "i", 0, 0},
	{"Alarm/Vebus", "TemperatureSenseError", 1, "i", 0, 0},
	{"Alarm/Vebus", "VoltageSenseError", 1, "i", 0, 0},
	{"Alarm/Vebus", "VeBusError", 1, "i", 0, 0},
	{"Relay", "Function", 0, "i", 0, 0},
	{"Relay", "Polarity", 0, "i", 0, 0},
	{"Gprs", "Enabled", 0, "i", 0, 0},
	{"Sensors/OnPosition", "ACIn1_L1", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACIn1_L2", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACIn1_L3", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACIn2_L1", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACIn2_L2", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACIn2_L3", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACOut_L1", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACOut_L2", "", "s", 0, 0},
	{"Sensors/OnPosition", "ACOut_L3", "", "s", 0, 0},
	{"Services", "Modbus", 0, "i", 0, 0},
	{"Services", "Mqtt", 0, "i", 0, 0},
	{"Services", "Console", 0, "i", 0, 0},
	{"Services", "LgResu", 0, "i", 0, 0},
	{"Services", "Vrmpubnub", 0, "i", 0, 0},
	{"Services", "OceanvoltMotorDrive", 0, "i", 0, 0},
	{"Services", "OceanvoltValence", 0, "i", 0, 0},
	{"Services", "VeCan", 1, "i", 0, 0}
};

const QString SETTINGS_SERVICE = "com.victronenergy.settings";

VSettings::VSettings(const QDBusConnection &connection, QObject *parent) :
	QObject(parent)
{
	m_addSettings = new VSettingsProxy(SETTINGS_SERVICE, "/Settings", connection, 0);
	addSettings();
}

void VSettings::addSettings()
{
	int numberOfSettings = sizeof(settings)/sizeof(SETTING);

	for (int i = 0; i < numberOfSettings; i++)
	{
		m_addSettings->AddSetting((QString)settings[i].group,
								  (QString)settings[i].name,
								  (QDBusVariant)settings[i].defaultValue,
								  (QString)settings[i].typeValue,
								  (QDBusVariant)settings[i].min,
								  (QDBusVariant)settings[i].max);
	}
}
