#include <QDeclarativeEngine>
#include <QtDeclarative>

#include <velib/qt/charger_error.hpp>

#include <platform/ve_platform.hpp>
#include <qmlapplicationviewer.h>
#include <ve_quick_view.hpp>

extern void initPlugin(QDeclarativeEngine *engine, QString themePAth = "");

VeQuickView::VeQuickView(VePlatform *platform, QWidget *parent) :
	QmlApplicationViewer(parent),
	mPlatform(platform),
	mLastKeyStillPressed(0),
	mWakeUpEvent(false)
{
	connect(engine(), SIGNAL(quit()), SLOT(close()));
	addImportPath(mPlatform->qmlUrl("qml"));
}

void VeQuickView::qmlRegister()
{
	initPlugin(engine(), mPlatform->themePath());
	engine()->addImportPath(mPlatform->qmlUrl("qt-components/qml"));

	rootContext()->setContextProperty("App", QCoreApplication::instance());

	qmlRegisterType<ChargerError>("com.victron.velib", 1, 0, "ChargerError");

	rootContext()->setContextProperty("NotificationCenter", &mNotificationCenter);
	rootContext()->setContextProperty("QuickView", this);

	mPlatform->qmlRegister(this);
}

void VeQuickView::loadApplication()
{
	setSource(mPlatform->qmlUrl("qml/main.qml"));
	//setSource(mPlatform->qmlUrl("qml/test.qml"));
	mPlatform->showView(this);
}

void VeQuickView::keyAccepted()
{
	mKeyAccepted = true;
}

void VeQuickView::keyPressEvent(QKeyEvent *event)
{
	/*
	 * When the display is enabled, a keypress causes resumption of normal
	 * operation. When a key is held long, repeated keys are send and interact
	 * with the menu, causing unwanted behviour. So ignore the wakeup key
	 * press and all its repeated keys.
	 */

	if (mWakeUpEvent)
		return;

	if (mPlatform->wakeupEvent()) {
		mWakeUpEvent = true;
		return;
	}

	if (mPlatform->silenceBuzzer())
		return;

	if (mLastKeyStillPressed == Qt::Key_Down && event->key() == Qt::Key_Up)
		takeSnapshot();

	mKeyAccepted = false;
	emit keyPressed(event->key());

	if (!mKeyAccepted)
			QDeclarativeView::keyPressEvent(event);

	mLastKeyStillPressed = event->key();
}

void VeQuickView::keyReleaseEvent(QKeyEvent *event)
{
	QDeclarativeView::keyReleaseEvent(event);
	mLastKeyStillPressed = 0;
	mWakeUpEvent = false;
}

void VeQuickView::takeSnapshot()
{
	QPixmap picture(QPixmap::grabWidget(this));
	QString fileName = QDir::current().filePath("snapshot.png");
	picture.save(fileName);
}
